# This Build.PL for Plack-Middleware-TrailingSlashKiller was generated by Dist::Zilla::Plugin::ModuleBuildTiny 0.007.
use strict;
use warnings;

use 5.008005;
use Module::Build::Tiny 0.038;
Build_PL();
